--
-- $Id: Template.hs,v 1.5 2005/12/16 14:41:28 aamine Exp $
--
-- Copyright (c) 2005 Minero Aoki
--
-- This program is free software.
-- You can distribute/modify this program under the terms of
-- the GNU LGPL, Lesser General Public License version 2.1.
-- For details of the GNU LGPL, see the file "COPYING".
--

module Template
    (TemplateRepository, fromConfig, fillTemplate) where

import Text.ParserCombinators.Parsec
import Data.List
import Data.Maybe
import Config

data TemplateRepository = TemplateRepository { prefix :: String }

fromConfig :: Config -> TemplateRepository
fromConfig conf =
    TemplateRepository { prefix = confLookupPath "directory" conf }

fillTemplate :: TemplateRepository -> String -> [(String,String)] -> IO String
fillTemplate repo id vals = do tmpl <- loadTemplate repo id
                               return $ fill tmpl vals

loadTemplate repo id = loadTemplate' id
  where
    loadTemplate' :: String -> IO String
    loadTemplate' id = procInclude =<< readTemplate id

    readTemplate :: String -> IO String
    readTemplate id = readFile (prefix repo ++ "/" ++ id)

    procInclude :: String -> IO String
    procInclude cs = return . unlines =<< procInclude' (lines cs)

    procInclude' :: [String] -> IO [String]
    procInclude' []     = return []
    procInclude' (s:xs) = do ss <- expand s
                             xs' <- procInclude' xs
                             return (ss ++ xs')

    expand :: String -> IO [String]
    expand s = if ".include " `isPrefixOf` s then
                   procInclude' . lines =<< readTemplate (words s !! 1)
               else
                   return [s]

fill tmpl vals = case (parse template "" tmpl) of
                   Right str -> str
                   Left err  -> "Template error"
  where
    template = return . concat =<< many component

    component =  many1 (noneOf "$")
             <|> do try (char '$')
                    name <- many1 alphaNum
                    return (expand name)
             <|> do c <- char '$'
                    return [c]

    expand name = fromMaybe ("$" ++ name) (lookup name vals)
